@@ -143,12 +143,6 @@ class ShellParamValidationTest(TestCase):
         ('stack-update', dict(
             command='stack-update ts -P "a-b"',
             err='Malformed parameter')),
-        ('validate', dict(
-            command='validate -P "a=b;c"',
-            err='Malformed parameter')),
-        ('template-validate', dict(
-            command='template-validate -P "a$b"',
-            err='Malformed parameter')),
     ]
 
     def setUp(self):
@@ -397,6 +391,8 @@ class ShellTestUserPass(ShellBase):
             'limit': 2,
             'status': ['COMPLETE', 'FAILED'],
             'marker': 'fake_id',
+            'global_tenant': True,
+            'show_deleted': 'True',
         }, True)
         fakes.script_heat_list(expected_url)
 
@@ -406,7 +402,9 @@ class ShellTestUserPass(ShellBase):
                                ' --limit 2'
                                ' --marker fake_id'
                                ' --filters=status=COMPLETE'
-                               ' --filters=status=FAILED')
+                               ' --filters=status=FAILED'
+                               ' --global-tenant'
+                               ' --show-deleted')
 
         required = [
             'teststack',
@@ -600,6 +598,11 @@ class ShellTestUserPass(ShellBase):
                     "output_key": "output2",
                     "description": "test output 2",
                 },
+                {
+                    "output_value": u"test\u2665",
+                    "output_key": "output_uni",
+                    "description": "test output unicode",
+                },
             ],
             "creation_time": "2012-10-25T01:58:47Z"
         }}
@@ -618,7 +621,7 @@ class ShellTestUserPass(ShellBase):
     def test_output_list(self):
         self._output_fake_response()
         list_text = self.shell('output-list teststack/1')
-        for r in ['output1', 'output2']:
+        for r in ['output1', 'output2', 'output_uni']:
             self.assertRegexpMatches(list_text, r)
 
     def test_output_show(self):
@@ -626,6 +629,11 @@ class ShellTestUserPass(ShellBase):
         list_text = self.shell('output-show teststack/1 output1')
         self.assertRegexpMatches(list_text, 'value1')
 
+    def test_output_show_unicode(self):
+        self._output_fake_response()
+        list_text = self.shell('output-show teststack/1 output_uni')
+        self.assertRegexpMatches(list_text, u'test\u2665')
+
     def test_template_show_cfn(self):
         self._script_keystone_client()
         template_data = open(os.path.join(TEST_VAR_DIR,
@@ -653,6 +661,36 @@ class ShellTestUserPass(ShellBase):
         for r in required:
             self.assertRegexpMatches(show_text, r)
 
+    def test_template_show_cfn_unicode(self):
+        self._script_keystone_client()
+        resp_dict = {"AWSTemplateFormatVersion": "2010-09-09",
+                     "Description": u"test\u2665",
+                     "Outputs": {},
+                     "Resources": {},
+                     "Parameters": {}}
+        resp = fakes.FakeHTTPResponse(
+            200,
+            'OK',
+            {'content-type': 'application/json'},
+            jsonutils.dumps(resp_dict))
+        http.HTTPClient.json_request(
+            'GET', '/stacks/teststack/template').AndReturn((resp, resp_dict))
+
+        self.m.ReplayAll()
+
+        show_text = self.shell('template-show teststack')
+        required = [
+            '{',
+            '  "AWSTemplateFormatVersion": "2010-09-09"',
+            '  "Outputs": {}',
+            '  "Parameters": {}',
+            u'  "Description": "test\u2665"',
+            '  "Resources": {}',
+            '}'
+        ]
+        for r in required:
+            self.assertRegexpMatches(show_text, r)
+
     def test_template_show_hot(self):
         self._script_keystone_client()
         resp_dict = {"heat_template_version": "2013-05-23",
@@ -818,7 +856,8 @@ class ShellTestUserPass(ShellBase):
                            'LinuxDistribution': 'F17"',
                            '"InstanceType': 'm1.large',
                            'DBPassword': 'verybadpassword'},
-            'timeout_mins': 123}
+            'timeout_mins': 123,
+            'disable_rollback': True}
         http.HTTPClient.json_request(
             'PUT', '/stacks/teststack2/2',
             data=expected_data,
@@ -997,6 +1036,7 @@ class ShellTestUserPass(ShellBase):
         update_text = self.shell(
             'stack-update teststack2/2 '
             '--template-file=%s '
+            '--enable-rollback '
             '--parameters="InstanceType=m1.large;DBUsername=wp;'
             'DBPassword=verybadpassword;KeyName=heat_key;'
             'LinuxDistribution=F17"' % template_file)
@@ -1271,21 +1311,22 @@ class ShellTestResources(ShellBase):
     def _script_keystone_client(self):
         fakes.script_keystone_client()
 
-    def test_resource_list(self):
+    def _test_resource_list(self, with_resource_name):
         self._script_keystone_client()
         resp_dict = {"resources": [
                      {"links": [{"href": "http://heat.example.com:8004/foo",
                                  "rel": "self"},
                                 {"href": "http://heat.example.com:8004/foo2",
                                  "rel": "resource"}],
-                      "logical_resource_id": "aResource",
+                      "logical_resource_id": "aLogicalResource",
                       "physical_resource_id":
                       "43b68bae-ed5d-4aed-a99f-0b3d39c2418a",
-                      "resource_name": "aResource",
                       "resource_status": "CREATE_COMPLETE",
                       "resource_status_reason": "state changed",
                       "resource_type": "OS::Nova::Server",
                       "updated_time": "2014-01-06T16:14:26Z"}]}
+        if with_resource_name:
+            resp_dict["resources"][0]["resource_name"] = "aResource"
         resp = fakes.FakeHTTPResponse(
             200,
             'OK',
@@ -1301,18 +1342,53 @@ class ShellTestResources(ShellBase):
         resource_list_text = self.shell('resource-list {0}'.format(stack_id))
 
         required = [
-            'resource_name',
             'resource_type',
             'resource_status',
             'updated_time',
-            'aResource',
             'OS::Nova::Server',
             'CREATE_COMPLETE',
             '2014-01-06T16:14:26Z'
         ]
+        if with_resource_name:
+            required.append('resource_name')
+            required.append('aResource')
+        else:
+            required.append('logical_resource_id')
+            required.append("aLogicalResource")
+
         for r in required:
             self.assertRegexpMatches(resource_list_text, r)
 
+    def test_resource_list(self):
+        self._test_resource_list(True)
+
+    def test_resource_list_no_resource_name(self):
+        self._test_resource_list(False)
+
+    def test_resource_list_empty(self):
+        self._script_keystone_client()
+        resp_dict = {"resources": []}
+        resp = fakes.FakeHTTPResponse(
+            200,
+            'OK',
+            {'content-type': 'application/json'},
+            jsonutils.dumps(resp_dict))
+        stack_id = 'teststack/1'
+        http.HTTPClient.json_request(
+            'GET', '/stacks/%s/resources' % (
+                stack_id)).AndReturn((resp, resp_dict))
+
+        self.m.ReplayAll()
+
+        resource_list_text = self.shell('resource-list {0}'.format(stack_id))
+
+        self.assertEqual('''\
++---------------+---------------+-----------------+--------------+
+| resource_name | resource_type | resource_status | updated_time |
++---------------+---------------+-----------------+--------------+
++---------------+---------------+-----------------+--------------+
+''', resource_list_text)
+
     def test_resource_show(self):
         self._script_keystone_client()
         resp_dict = {"resource":
